//================================================================================
//  PanelInfo.java
//--------------------------------------------------------------------------------
// Contains information about the part of the trace visible in the window
// and the relationship between samples and pixels in the display. 
//
//================================================================================
//
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//       1         2         3         4         5         6         7         8

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.lang.*;
import Trace;


//------------------------------------------------------------------------------
// * PanelInfo
//------------------------------------------------------------------------------
// Contains information about the part of the trace visible in the window
// and the relationship between samples and pixels in the display. 
//
class PanelInfo {
	protected int A=0,C=1,G=2,T=3,N=4;
	int winSampleStart=0; 	// The first sample visible in viewing window (IN)	
	int winVisibleExponet; 	// Power of 2 fraction of the trace visible (IN)
	int vScale=1;			// Vertical scale (IN)
	int TotalSamples;		// Total number of samples in the trace (IN)
	
	int winSampleEnd;		// The last sample visible in viewing window (calc)
	double winSampleWidth; 	// The number of samples visible in viewing win (calc)

	double fraction;		// KJD Bug

	protected Color baseColor[]= new Color[5];

//------------------------------------------------------
// * PanelInfo (PanelInfo)
//------------------------------------------------------
//
PanelInfo(int TraceLength,int exponet){

	TotalSamples = TraceLength;
	winVisibleExponet = exponet;
	fraction = (1.0/Math.pow((double)2,(double)winVisibleExponet/2.0));
 	winSampleWidth = fraction * TotalSamples;
 	winSampleEnd = winSampleStart+(int)winSampleWidth;
  	if (winSampleEnd >=TotalSamples){
  		winSampleEnd = TotalSamples-1;
  	}
  
  	// Color table. 
  	Color stdGreen = Color.green;
	baseColor[A]= stdGreen.darker();	// A
	baseColor[C]= Color.blue;			// C
	baseColor[G]= Color.black;			// G
	baseColor[T]= Color.red;			// T
	baseColor[N]= Color.gray;			// N
}



//------------------------------------------------------
// * SetVisibleFraction (PanelInfo)
//------------------------------------------------------
//
public void SetVisibleFraction(int exponet)
{
	winVisibleExponet = exponet;
	fraction = (1.0/Math.pow((double)2,(double)winVisibleExponet/2.0));
	winSampleWidth = fraction * TotalSamples;
	winSampleEnd = winSampleStart+(int)winSampleWidth;
	if (winSampleEnd >=TotalSamples){
  		winSampleEnd = TotalSamples-1;
  	}
}

//------------------------------------------------------
// * SetWinSampleStart (PanelInfo)
//------------------------------------------------------
//
public void SetWinSampleStart(int start)
{
	winSampleStart = start;
	winSampleEnd = winSampleStart+(int)winSampleWidth;
  	if (winSampleEnd >=TotalSamples){
  		winSampleEnd = TotalSamples-1;
  	}
}
	
//------------------------------------------------------
// * SetVScale (PanelInfo)
//------------------------------------------------------
//
public void SetVScale(int scale){
	vScale = scale;
}

//------------------------------------------------------
// * SetTotalSamples
//------------------------------------------------------
//
public void SetTotalSamples(int TraceLength){
	TotalSamples = TraceLength;
	fraction = (1.0/Math.pow((double)2,(double)winVisibleExponet/2.0));
 	winSampleWidth = fraction * TotalSamples;
 	winSampleEnd = winSampleStart+(int)winSampleWidth;
  	if (winSampleEnd >=TotalSamples){
  		winSampleEnd = TotalSamples-1;
  	}
}


}// ---------| End class PanelInfo |---------------------------------------------


