//================================================================================
//  SCFHeader.java
//--------------------------------------------------------------------------------
//
//--------------------------------------------------------------------------------
// SCFTrace and ABITrace are adapted from similar classes in:
// Trace Applet, by Eugen Buehler.   Release 1, 6/30/96 
//
// Adapted by K. James Durbin, Baylor College of Medicine.  7/17/97
//================================================================================
//
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//       1         2         3         4         5         6         7         8
//
import java.awt.*;
import java.io.*;
import Trace;

//------------------------------------------------------------------------------
// * SCFHeader
//------------------------------------------------------------------------------
// Encapsulation and description of the format of the SCF Header.  Note that 
// some variables are unused but are shown as comments for reference to the file
// orginization which mirrors the order in this structure.   
//
class SCFHeader {

	//public static final int
	// MagicNum = (((((int)'.'<<8)+(int)'s'<<8)+(int)'c'<<8)+(int)'f');
	public int sampleNum;			// Number of elements in Samples matrix
	public int SamplesOffset;		// Byte offset from start of file. 
	public int baseNum;				// Number of bases in bases matrix.
	//public int bases_left_clip;	// OBSOLETE
	//public int bases_right_clip;// OBSOLETE
	public int basesOffset;			// Byte offset from start of file.
	public int commentsSize; 		// Number of bytes in comment section.
	public int commentsOffset;		// Byte offset from start of file. 
	public float versionNum;		// version.revision
	public int sampleSize;			// Size of samples in bytes 1= 8 bits, 2 = 16 bits. 
	public int codeSet;				// Code set used. 
	// public int private_size		// Number of bytes of private data, 0 if none.
	// public int spare				// Unused
	
	//  versionString is a local variable, it is not in SCF file. 
	String 	versionString = "";

	//------------------------------------------------------
	// * Read  (SCFHeader)
	//------------------------------------------------------
	//
	public int read(DataInputStream filterIN) throws IOException
	{	
		filterIN.skipBytes(4);					// MagicNum
		sampleNum = filterIN.readInt();
		SamplesOffset = filterIN.readInt();
		baseNum = filterIN.readInt();
		filterIN.skip(8);							// bases_left_clip & bases_right_clip
		basesOffset = filterIN.readInt();
		commentsSize = filterIN.readInt();
		commentsOffset = filterIN.readInt();
	
		// read the Version Number and put into a float.
		for (int i=0;i<4;i++) {
			versionString += (char) filterIN.readUnsignedByte();
		}
		versionNum = Float.valueOf(versionString).floatValue();
	
		sampleSize = filterIN.readInt();
		codeSet = filterIN.readInt();
		
		return (48);								// Total number of bytes read.
	}
}
