//================================================================================
// TraceCanvas
//--------------------------------------------------------------------------------
//
//  Canvas where trace, bases, and base locations are painted. 
//
//================================================================================
//
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//       1         2         3         4         5         6         7         8
import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import Trace;
import PanelInfo;


//------------------------------------------------------------------------------
// * TraceCanvas
//------------------------------------------------------------------------------
//
class TraceCanvas extends Canvas {
Trace mTrace;
int mScaledTrace[][];
PanelInfo mPanelInfo;
boolean mLine=true,mTick=true,mA=true,mC=true,mG=true,mT=true;
boolean mInvalid = false;

double PixelsPerSample;
int xLookup[];

// Variables for offscreen graphics manipulation. 
protected Dimension offDimension;
protected Graphics offGraphics;
protected Image offImage;

// Note: This is not good, since there is another copy of the next few 
// data elements.  Consolidate.... KJD
protected int A=0,C=1,G=2,T=3,N=4;
	
// To display bases these numbers need to be converted back to letters 
// as follows
protected char baseLetterFromNum[]={'A','C','G','T','N'};
	
	// Color table. 
protected Color baseColor[]= new Color[5];
protected Color baseColorA,baseColorC,baseColorG,baseColorT,baseColorN;

	
//------------------------------------------------------
// * TraceCanvas
//------------------------------------------------------
//
TraceCanvas(Trace theTrace,PanelInfo theInfo){

	mTrace=theTrace;
	
	mPanelInfo = theInfo;	
		
	// We make a local copy of the color table for efficiency, to avoid
	// another dereference in the inner loop. 
	baseColor[A]= mPanelInfo.baseColor[A];	// A
	baseColor[C]= mPanelInfo.baseColor[C];	// C
	baseColor[G]= mPanelInfo.baseColor[G];	// G
	baseColor[T]= mPanelInfo.baseColor[T];	// T
	baseColor[N]= mPanelInfo.baseColor[N];	// N
	
	// In an effort to squeeze every bit of performance out of the inner 
	// plot Lines loop, these variables are defined again here so that
	// they can be used without the class and array dereferences. 
	baseColorA= mPanelInfo.baseColor[A];	// A
	baseColorC= mPanelInfo.baseColor[C];	// C
	baseColorG= mPanelInfo.baseColor[G];	// G
	baseColorT= mPanelInfo.baseColor[T];	// T
	baseColorN= mPanelInfo.baseColor[N];	// N
}

public Dimension minimumSize(){
	return (new Dimension(200,50));
}

public Dimension preferredSize(){
	return (new Dimension(600,100));
}


//------------------------------------------------------
// * SetNewTrace
//------------------------------------------------------
//
public void SetNewTrace (Trace theTrace)
{
	mTrace = theTrace;
	InitScaledTraceDataY();
	InitScaledTraceDataX();
	mInvalid = true;
}

//------------------------------------------------------
// * FinishInit
//------------------------------------------------------
// Performs initialization that depends on the screen size
// which is not available until the componet this is in 
// has been validated.  
//
public void FinishInit()
{
	InitScaledTraceDataX();
		
	// Create a local copy of the trace data which is scaled according
	// to the setting of the scroll bar.  This is a performance hit when
	// loading but will provide some efficiency gains in drawng and when
	// changing the scale.
	InitScaledTraceDataY();
	show();
	repaint();
}

//------------------------------------------------------
// * setLine
//------------------------------------------------------
//
public void setLine(boolean state){
	mLine = state;
}

//------------------------------------------------------
// * setTick
//------------------------------------------------------
//
public void setTick(boolean state){
	mTick = state;
}
//------------------------------------------------------
// * setA
//------------------------------------------------------
//
public void setA(boolean state){
	mA = state;
}
//------------------------------------------------------
// * setC
//------------------------------------------------------
//
public void setC(boolean state){
	mC = state;
}
//------------------------------------------------------
// * setG
//------------------------------------------------------
//
public void setG(boolean state){
	mG = state;
}
//------------------------------------------------------
// * setT
//------------------------------------------------------
//
public void setT(boolean state){
	mT = state;
}

//------------------------------------------------------
// * InitScaledTraceDataY
//------------------------------------------------------
// Create a local scaled copy ofthe trace data. 
//
public void InitScaledTraceDataY()
{
	mScaledTrace = new int[4][mTrace.TraceData[A].length];
	
	//System.out.println("Allocate scaled Array complete.");
	
	System.arraycopy(mTrace.TraceData[A],0,mScaledTrace[A],0,
						  mTrace.TraceData[A].length);
	
	System.arraycopy(mTrace.TraceData[C],0,mScaledTrace[C],0,
						  mTrace.TraceData[C].length);
					  
	System.arraycopy(mTrace.TraceData[G],0,mScaledTrace[G],0,
						  mTrace.TraceData[G].length);

	System.arraycopy(mTrace.TraceData[T],0,mScaledTrace[T],0,
						  mTrace.TraceData[T].length);
	  						  	
	RescaleTraceDataY();
}

//------------------------------------------------------
// * InitScaledTraceDataX
//------------------------------------------------------
// Create a local scaled copy ofthe trace data. 
//
public void InitScaledTraceDataX()
{
	xLookup = new int[mPanelInfo.TotalSamples];
	RescaleTraceDataX();
}

//------------------------------------------------------
// * RescaleTraceDataX
//------------------------------------------------------
// Updates the lookup table that stores pixel locations 
// given sample numbers relative to the viewing window. 
// This is also aimed at performance by moving this 
// multiply outside of the drawing loop.  
//
public void RescaleTraceDataX(){
	PixelsPerSample = ((double)size().width/(double)mPanelInfo.winSampleWidth);
	//System.out.println("visibleExponent="+mPanelInfo.winVisibleExponet+" TotalSamples="+
	//mPanelInfo.TotalSamples);
	//System.out.println("size.width="+size().width+" winSampleWidth ="+
	//mPanelInfo.winSampleWidth);
	//System.out.println("fraction="+mPanelInfo.fraction);
	//System.out.println("Rescale: PixPerSamp="+PixelsPerSample);
	//System.out.println("TotalSamples="+mPanelInfo.TotalSamples);
	for (int i = 0;i<mPanelInfo.TotalSamples;i++){
		xLookup[i]=(int)(i*PixelsPerSample);
	}
}

//------------------------------------------------------
// * RescaleTraceDataY
//------------------------------------------------------
// Scales the local copy of the trace data according to the 
// current setting of the vScale scroll bar.  Scalling all of
// the data at once like this is a performance hit when 
// manipulating the vertical scroll bar, but improves the 
// performance when scrolling, which is done more frequently. 
// The original copy of the trace data is left unaltered.  
//
public void RescaleTraceDataY()
{
	int Index;
	double adjustedVScale;
	//System.out.print("Rescaling trace data...");	
	
	adjustedVScale = mPanelInfo.vScale/100.0;

	for(Index = 0;Index < mTrace.TraceData[A].length;Index++){
		mScaledTrace[A][Index] =(int)(mTrace.TraceData[A][Index]*adjustedVScale);
		mScaledTrace[C][Index] =(int)(mTrace.TraceData[C][Index]*adjustedVScale);
		mScaledTrace[G][Index] =(int)(mTrace.TraceData[G][Index]*adjustedVScale);
		mScaledTrace[T][Index] =(int)(mTrace.TraceData[T][Index]*adjustedVScale);
	}
	//System.out.println("done.");
}

//------------------------------------------------------
// * PlotPoints
//------------------------------------------------------
// See performance comments in PlotLines.  Drawing points is
// probably only useful if you want to see where the actual 
// samples are and get a feel for the number of samples in a
// region.  It's kind of hard to see otherwise.      
//
public void PlotPoints(Graphics g)
{
	int SampleIndex,height;
	int x1,y1,SampleStart,SampleEnd;

	height = size().height-3;
   
   SampleStart = mPanelInfo.winSampleStart;
   SampleEnd = mPanelInfo.winSampleEnd;
   if (mA == true){
   	g.setColor(baseColorA);
 		for (SampleIndex = SampleStart; SampleIndex< SampleEnd;SampleIndex++){
 			// As a performance measure, lookup table replaces the following 
 			// statement: 
			// x1 = (int)((SampleIndex - SampleStart)*PixelsPerSample);
			x1 = xLookup[SampleIndex-SampleStart];
   		y1=(int)(height-mScaledTrace[A][SampleIndex]);
			g.drawRect(x1,y1,1,1);
		}
	}
 	if (mC == true){
 		g.setColor(baseColorC);
 		for (SampleIndex = SampleStart; SampleIndex< SampleEnd;SampleIndex++){
 			x1 = xLookup[SampleIndex-SampleStart];
   		y1=(int)(height-mScaledTrace[C][SampleIndex]);
			g.drawRect(x1,y1,1,1);
		}
	}
	if (mG == true){
		g.setColor(baseColorG);
 		for (SampleIndex = SampleStart; SampleIndex< SampleEnd;SampleIndex++){
 			x1 = xLookup[SampleIndex-SampleStart];
   		y1=(int)(height-mScaledTrace[G][SampleIndex]);
			g.drawRect(x1,y1,1,1);
		}
	}
	if (mT == true){
		g.setColor(baseColorT);
 		for (SampleIndex = SampleStart; SampleIndex< SampleEnd;SampleIndex++){
 			x1 = xLookup[SampleIndex-SampleStart];
   		y1=(int)(height-mScaledTrace[T][SampleIndex]);
			g.drawRect(x1,y1,1,1);
		}
	}
}	



//------------------------------------------------------
// * PlotLines
//------------------------------------------------------
// Performance is critical here.  My choice to keep a separate
// scaled copy of the data was aimed at reducing time inside 
// this function.  Drawing time can be further reduced by 
// pre-adding the height but beyond that I can see few additional
// opportunities for speeding this up.  If the visual effect
// is tollerable, we could try to plot only every other point. 
// That would cut drawing time in half.  Perhaps as a user 
// option?   
//
public void PlotLines(Graphics g)
{
	int SampleIndex,height;
	int x1,x2,y1,y2,SampleStart,SampleEnd;

	height = size().height-3;
   
   SampleStart = mPanelInfo.winSampleStart;
   SampleEnd = mPanelInfo.winSampleEnd;
   if (mA==true){
   	g.setColor(baseColorA);
 		for (SampleIndex = SampleStart; SampleIndex< SampleEnd;SampleIndex++){
 		
 			// As a performance measure, lookup table replaces the following 
 			// statement: 
			// x1 = (int)((SampleIndex - SampleStart)*PixelsPerSample);
			x1 = xLookup[SampleIndex-SampleStart];
		
			// (i.e. (Index+1-winSampleStart)*PixelsPerSample
   		x2 = (int)(x1+PixelsPerSample);
   
   		y1=(int)(height-mScaledTrace[A][SampleIndex]);
   		y2=(int)(height-mScaledTrace[A][SampleIndex+1]);
			g.drawLine(x1,y1,x2,y2);
		}
	}
	if (mC==true){
		g.setColor(baseColorC);
		for (SampleIndex = SampleStart; SampleIndex< SampleEnd;SampleIndex++){
			x1 = xLookup[SampleIndex-SampleStart];
   		x2 = (int)(x1+PixelsPerSample);
   		y1=(int)(height-mScaledTrace[C][SampleIndex]);
   		y2=(int)(height-mScaledTrace[C][SampleIndex+1]);
			g.drawLine(x1,y1,x2,y2);
		}
	}
	if (mG==true){
		g.setColor(baseColorG);
		for (SampleIndex = SampleStart; SampleIndex< SampleEnd;SampleIndex++){
			x1 = xLookup[SampleIndex-SampleStart];
   		x2 = (int)(x1+PixelsPerSample);
   		y1=(int)(height-mScaledTrace[G][SampleIndex]);
   		y2=(int)(height-mScaledTrace[G][SampleIndex+1]);
			g.drawLine(x1,y1,x2,y2);
		}
	}
	if (mT==true){
		g.setColor(baseColorT);
		for (SampleIndex = SampleStart; SampleIndex< SampleEnd;SampleIndex++){
			x1 = xLookup[SampleIndex-SampleStart];
   		x2 = (int)(x1+PixelsPerSample);
   		y1=(int)(height-mScaledTrace[T][SampleIndex]);
   		y2=(int)(height-mScaledTrace[T][SampleIndex+1]);
			g.drawLine(x1,y1,x2,y2);
		}
	}
}	

//------------------------------------------------------
// * PlotBasesWithTicks
//------------------------------------------------------
// Plots base names, tick marks, and base locations. 
// Performance issues also loom here, see comments in
// PlotLines(). 
//
public void PlotBasesWithTicks(Graphics g)
{
	int    x,y,thePosition=0,TickHeight,BaseHeight,PosHeight,start;
	int	 theBase,fontOffset;
	
	// We'll let G be the canonical letter width for all letters... close enough
	// and saves the time to compute it each time through the loop...
	int LetterCenter = getFontMetrics(g.getFont()).charWidth('G') / 2;
	BaseHeight = getFontMetrics(g.getFont()).getHeight()-2;
	PosHeight = BaseHeight+BaseHeight;
   TickHeight = size().height-10;
	start = mPanelInfo.winSampleStart;
	
	// Find the first and last base within our window.  
	int BaseStart = mTrace.GetNextBaseAfterSample(start);
	int BaseEnd = mTrace.GetLastBaseBeforeSample(mPanelInfo.winSampleEnd);
 	//System.out.println("BaseStart="+BaseStart+" BaseEnd="+BaseEnd);
 	
 try{
 
    // Draw all the base letters, tick marks, and base positions in that range.
  	for (int Index=BaseStart;Index <= BaseEnd;Index++) {
		theBase = mTrace.getBase(Index);
		thePosition = mTrace.getBasePosition(Index);
		
		g.setColor(mPanelInfo.baseColor[theBase]);
		
		x = (int)((thePosition - start)* PixelsPerSample);
		
		// Draw base
		g.drawString(String.valueOf(baseLetterFromNum[theBase]),x-LetterCenter,
		BaseHeight);
	
		// Draw tick mark.  N's have no ticks. 	
		if (theBase != N){
			y = TickHeight-mScaledTrace[theBase][thePosition];		
			g.drawLine(x,y,x,y-5);
		}
						 
		// Draw base position
		if ((Index % 10) == 0) {
			g.setColor(Color.black);
			fontOffset = getFontMetrics(g.getFont()).stringWidth(String.valueOf(Index))/2;
			x = (int)((thePosition - mPanelInfo.winSampleStart)*PixelsPerSample);
			g.drawString(String.valueOf(Index),x - fontOffset,PosHeight);
		}
	}
}
catch (ArrayIndexOutOfBoundsException ex){
		String exMessage = ex.getMessage();
		String exName = ex.getClass().getName();
		System.out.println("Exception: " + exName + " : " + exMessage);
		System.out.println("Final position="+thePosition);
}
}

//------------------------------------------------------
// * PlotBases
//------------------------------------------------------
// Plots base names only.  This is made a whole separate
// function since, as part of my all out assault on speed, 
// I didn't want to put the mode test inside the loop.  
//
public void PlotBases(Graphics g)
{
	int    x,thePosition,BaseHeight,PosHeight,start;
	int	 theBase,fontOffset;
	
	// We'll let G be the canonical letter width for all letters... close enough
	// and saves the time to compute it each time through the loop...
	int LetterCenter = getFontMetrics(g.getFont()).charWidth('G') / 2;
	BaseHeight = getFontMetrics(g.getFont()).getHeight()-2;
	PosHeight = BaseHeight+BaseHeight;
	start = mPanelInfo.winSampleStart;
	
	// Find the first and last base within our window.  
	int BaseStart = mTrace.GetNextBaseAfterSample(start);
	int BaseEnd = mTrace.GetLastBaseBeforeSample(mPanelInfo.winSampleEnd);
 
    // Draw all the base letters, tick marks, and base positions in that range.
  	for (int Index=BaseStart;Index <= BaseEnd;Index++) {
		theBase = mTrace.getBase(Index);
		thePosition = mTrace.getBasePosition(Index);
		
		g.setColor(mPanelInfo.baseColor[theBase]);
		
		x = (int)((thePosition - start)* PixelsPerSample);
		
		// Draw base
		g.drawString(String.valueOf(baseLetterFromNum[theBase]),x-LetterCenter,BaseHeight);
						 
		// Draw base position
		if ((Index % 10) == 0) {
			g.setColor(Color.black);
			fontOffset = getFontMetrics(g.getFont()).stringWidth(String.valueOf(Index))/2;
			x = (int)((thePosition - mPanelInfo.winSampleStart)*PixelsPerSample);
			g.drawString(String.valueOf(Index),x - fontOffset,PosHeight);
		}
	}
}

		
//------------------------------------------------------
// * paintFrame
//------------------------------------------------------
// Called whenever something alters the canvas.  
//
public void paintFrame(Graphics g) {
	int Index;
	
	PixelsPerSample = ((double)size().width/mPanelInfo.winSampleWidth);
  	  
  	if (mLine){
   	PlotLines(g);
   }
   else{
   	PlotPoints(g);
   }
   
   if (mTick){
   	PlotBasesWithTicks(g);
   }
   else{
   	PlotBases(g);
   }
}

//------------------------------------------------------
// * update
//------------------------------------------------------
// Reduce flicker with double buffering, drawing the display in 
// an offscreen window of the same size and slaping the whole thing 
// up at once.  On systems with really slow drawing this may 
// lead to unacceptable delays w/o feedback after a user clicks 
// on a control.  For such systems this whole method may be eliminated
// and paintFrame() changed to paint() so that the user can see the
// drawing take place as feedback for mouse clicks...
public void update(Graphics g){

	//System.out.println("Canvas update");
	Dimension d = size();
	
	// create offscreen graphics context. 
	if ((offGraphics == null) ||
		(d.width != offDimension.width) ||
		(d.height != offDimension.height)){
		offDimension = d;
		offImage = createImage(d.width,d.height);
		offGraphics = offImage.getGraphics();
		offGraphics.setColor(Color.black);
	}
	
	//  Clear the previous frame
	offGraphics.setColor(getBackground());
	offGraphics.fillRect(0,0,d.width,d.height);
	
	// Paint the frame into the image.
	paintFrame(offGraphics);
			
	// Paint the image onto the screen.
	g.drawImage(offImage,0,0,null);			
}

//------------------------------------------------------
// * paint
//------------------------------------------------------
// Puts the previously drawn offscreen image onto the screen. 
// repaint() does not reliably call update (at least, I can not see 
// the factors that determine when it happens), but it does seem
// to reliably call either paint or update.  So there is a hook 
// here to call update if the offscreen image has not been created.
// We also have hooks detect and handle a screen size change. 
//
public void paint(Graphics g){
	Dimension d = size();
	
	// If this is the first time, update the display..
	if (offImage == null){
		update(g);
		return;
	}
	
	if (mInvalid){
		//System.out.println("TraceCanvas.paint:mInvalid");
		mInvalid = false;
		update(g);
		return;
	}
	
	// If the size has changed, rescale the axes and update the display...
	if	((d.width != offDimension.width) ||
		(d.height != offDimension.height)){
		
		if (d.width != offDimension.width){
			RescaleTraceDataX();
		}
		if (d.height != offDimension.height){
			RescaleTraceDataY();
		}
		update(g);
		return;
	}
	
	// If the contents haven't changed somehow, just repaint previous image. 
	g.drawImage(offImage,0,0,null);
}



}//------| End TraceCanvas |------------------------------------------------
 