//================================================================================
//  TracePanel.java
//--------------------------------------------------------------------------------
// Encapsulates the interface portion of viewing a trace.  Multiple traces 
// may be viewed within a single applet by instantiating multiple copies of 
// this class (passing in different traces).   
//
//================================================================================
//
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//       1         2         3         4         5         6         7         8

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import Trace;
import PanelInfo;
import TraceCanvas;
import QualityCanvas;

//------------------------------------------------------------------------------
// * TracePanel
//------------------------------------------------------------------------------
//  This is a componet which contains all of the graphics obects. 
//  This encapsulation means that displaying multiple traces within an applet
//  requires only that multiple instances of TracePanel be created.  
//
public class TracePanel extends Panel{
Scrollbar 		hScroll,vScale,hScale;
TraceCanvas 	theTraceCanvas;
QualityCanvas	theQualityCanvas;
Checkbox		LineCheck,TickCheck,GraphCheck,TempCheck,ACheck,CCheck;
Checkbox		GCheck,TCheck;
Panel			CheckBoxPanel;

Trace			mTrace;
PanelInfo    	mPanelInfo;
boolean	    	mInvalid = false;

//------------------------------------------------------
// * TracePanel (TracePanel)
//------------------------------------------------------
//
TracePanel(Trace theTrace)
{
	mTrace = theTrace;
	
	// Setup Display
	setBackground(Color.white);
	
	// Scrollbar for the horizontal scale. 
	// Arguments: value, visible, min, max
	int winFractionPower = 9;
	hScale = new Scrollbar(Scrollbar.VERTICAL,winFractionPower,1,1,18);
	
	// Object to manage window size parameters.
	mPanelInfo = new PanelInfo(mTrace.getTraceLength(),winFractionPower);
	
	// Scrollbar for the vertical scale. 
	// Arguments: value, visible, min, max
	vScale = new Scrollbar(Scrollbar.VERTICAL,7,2,1,15);
	vScale.setPageIncrement(2);
	mPanelInfo.SetVScale(vScale.getValue());
			
	// Scrollbar for the position within the trace.  
	int width = (int) mPanelInfo.winSampleWidth;
	hScroll= new Scrollbar(Scrollbar.HORIZONTAL,1,width,1,
							mPanelInfo.TotalSamples);
							
	hScroll.setPageIncrement(width);
	
	theTraceCanvas = new TraceCanvas(mTrace,mPanelInfo);
	theQualityCanvas = new QualityCanvas(theTrace,mPanelInfo);
	
	SetupCheckBoxes();
	
	SetupLayout();								   		
	
	validate();	
}

//------------------------------------------------------
// * FinishInit
//------------------------------------------------------
// Performs initializations that depend on screen sizes and are not 
// defined at the time the constructor is called. 
//
public void FinishInit()
{
	theTraceCanvas.FinishInit();
	theQualityCanvas.FinishInit();
	this.show();
}

//------------------------------------------------------
// * insets
//------------------------------------------------------
// Ensure that no componet is placed on top of the border
public Insets insets(){
	return new Insets(8,8,8,8);
}

//------------------------------------------------------
// * paintBorder
//------------------------------------------------------
// Paint border around view. 
public void paintBorder(Graphics g){
	Dimension d = size();
	g.setColor(Color.darkGray);
	g.draw3DRect(0,0,d.width-1,d.height-1,true);
	g.draw3DRect(3,3,d.width-7,d.height-7,false);
}



//------------------------------------------------------
// * paint
//------------------------------------------------------
//  
public void paint(Graphics g){
	 //System.out.println("TracePanel.paint");
	 if (mInvalid){
		  update(g);
		  mInvalid = false;
	 }
	 paintBorder(g);
}

//------------------------------------------------------
// * update
//------------------------------------------------------
//
public void update(Graphics g){
	//System.out.println("TracePanel.update");
	paintBorder(g);
	theTraceCanvas.repaint();
	theQualityCanvas.repaint();
}

//------------------------------------------------------
// * setNewTrace
//------------------------------------------------------
// Puts a new trace in the panel.  
//
public void setNewTrace(Trace newTrace)
{
	//System.out.println("Setting new trace.  Old len="+
	//mTrace.getTraceLength()+" new len="+newTrace.getTraceLength());
	
	mTrace = newTrace;
	
	//System.out.println("After set new trace.  Old len="+
	//mTrace.getTraceLength()+" new len="+newTrace.getTraceLength());
	
	// Update the panel info...
	mPanelInfo.SetTotalSamples(mTrace.getTraceLength());
	mPanelInfo.SetWinSampleStart(0);

	// Update scroll bars...

	int width = (int) mPanelInfo.winSampleWidth;
	hScroll.setPageIncrement((int)mPanelInfo.winSampleWidth);
	hScroll.setMaximum(mPanelInfo.TotalSamples);
	hScroll.setValues(1,width,1,mPanelInfo.TotalSamples);

	theTraceCanvas.SetNewTrace(mTrace);
	theQualityCanvas.SetNewTrace(mTrace);
	
	// Mark this panel as invalid for purposes of repainting.
	mInvalid = true;
}

//------------------------------------------------------
// * SetupCheckBoxes
//------------------------------------------------------
//
public void SetupCheckBoxes()
{
	LineCheck= new  Checkbox("Lines");
	LineCheck.setState(true);
	TickCheck= new  Checkbox("Ticks");
	TickCheck.setState(true);
	GraphCheck= new Checkbox("Graph");
	GraphCheck.setState(true);
	TempCheck= new Checkbox("Temp");
	TempCheck.setState(true);
	
	// Create check boxes and color them.  
	ACheck = new Checkbox("A");
	ACheck.setState(true);
	ACheck.setForeground(mPanelInfo.baseColor[mPanelInfo.A]);
	CCheck = new Checkbox("C");
	CCheck.setState(true);
	CCheck.setForeground(mPanelInfo.baseColor[mPanelInfo.C]);
	GCheck = new Checkbox("G");
	GCheck.setState(true);
	GCheck.setForeground(mPanelInfo.baseColor[mPanelInfo.G]);
	TCheck = new Checkbox("T");
	TCheck.setState(true);
	TCheck.setForeground(mPanelInfo.baseColor[mPanelInfo.T]);

	// Define layout for checkboxes.
	CheckBoxPanel = new Panel();
	
	String constraints[] = {
		"LineCheck.top=form.top",
		"LineCheck.left=form.left",
		
		"TickCheck.top=LineCheck.bottom",
		"TickCheck.left=form.left",
		
		"GraphCheck.top=TickCheck.bottom",
		"GraphCheck.left=form.left",
		
		"TempCheck.top=GraphCheck.bottom",
		"TempCheck.left=form.left",
		
		"ACheck.top=TempCheck.bottom",
		"ACheck.left=form.left",
		
		"CCheck.top=ACheck.bottom",
		"CCheck.left=form.left",
		"GCheck.top=ACheck.top",
		"GCheck.left=ACheck.right",
		
		"TCheck.top=CCheck.top",
		"TCheck.left=ACheck.right"
	};
	
	CheckBoxPanel.setLayout(new XmFormLayout(constraints));
	CheckBoxPanel.add("LineCheck",LineCheck);
	CheckBoxPanel.add("TickCheck",TickCheck);
	CheckBoxPanel.add("GraphCheck",GraphCheck);
	CheckBoxPanel.add("TempCheck",TempCheck);
	CheckBoxPanel.add("ACheck",ACheck);
	CheckBoxPanel.add("CCheck",CCheck);
	CheckBoxPanel.add("GCheck",GCheck);
	CheckBoxPanel.add("TCheck",TCheck);
}

//------------------------------------------------------
// * SetupLayour
//------------------------------------------------------
// Has some absolute sizes that should be made font relative
//
public void SetupLayout()
{
	String constraints[] = {
		"hScale.left=form.left",
		"hScale.top=form.top",
		"hScale.bottom=hScroll.top",
		
		"hScroll.bottom=form.bottom",
		"hScroll.left=form.left",
		"hScroll.right=CheckBoxPanel.left",
		
		"CheckBoxPanel.top=form.top",
		"CheckBoxPanel.bottom=form.bottom",
		"CheckBoxPanel.right=form.right",
		"CheckBoxPanel.left=CheckBoxPanel.right-70",
		
		"vScale.top=form.top",
		"vScale.right=CheckBoxPanel.left",
		"vScale.bottom=hScroll.top",
		
		"theTraceCanvas.left=hScale.right",
		"theTraceCanvas.right=vScale.left",
		"theTraceCanvas.bottom=hScroll.top",
		"theTraceCanvas.top=QualityCanvas.bottom",
		
		"QualityCanvas.top=form.top",
		"QualityCanvas.left=hScale.right",
		"QualityCanvas.right=vScale.left"
	};

	this.setLayout(new XmFormLayout(constraints));
	
	this.add("CheckBoxPanel",CheckBoxPanel);
	this.add("QualityCanvas",theQualityCanvas);
	this.add("hScale",hScale);
	this.add("theTraceCanvas",theTraceCanvas);
	this.add("vScale",vScale);
	this.add("hScroll",hScroll);	
}


//------------------------------------------------------
// * handleEvent
//------------------------------------------------------
//
public boolean handleEvent(Event theEvent){
	int temp;
		
	switch (theEvent.id){

		case Event.MOUSE_UP:
			break;
			
		case Event.SCROLL_ABSOLUTE:	
			//System.out.println("Scroll absolute");
		case Event.SCROLL_LINE_UP:
		case Event.SCROLL_LINE_DOWN:
		case Event.SCROLL_PAGE_UP:
		case Event.SCROLL_PAGE_DOWN:
		
		if (theEvent.target == hScroll){
			temp = ((Integer)theEvent.arg).intValue();
			mPanelInfo.SetWinSampleStart(temp);
			theTraceCanvas.repaint();
			theQualityCanvas.repaint();
		}
		
		if (theEvent.target == vScale){
			temp = ((Integer)theEvent.arg).intValue();
			mPanelInfo.SetVScale(temp);
			theTraceCanvas.RescaleTraceDataY();
			theTraceCanvas.repaint();	
		}
		
		// update events are not generated in Netscape or we'd wait 
		// until MOUSE_UP before taking any action. 
		if (theEvent.target == hScale){
			temp = ((Integer)theEvent.arg).intValue();
			mPanelInfo.SetVisibleFraction(temp);

			int width = (int) mPanelInfo.winSampleWidth;
			hScroll.setPageIncrement((int)mPanelInfo.winSampleWidth);
			hScroll.setMaximum(mPanelInfo.TotalSamples);
			hScroll.setValues(1,width,1,mPanelInfo.TotalSamples);

			theTraceCanvas.RescaleTraceDataX();
			theTraceCanvas.repaint();
			theQualityCanvas.repaint();
		}
		return true;
	} 
	return super.handleEvent(theEvent);
}	

//------------------------------------------------------
// * action
//------------------------------------------------------
// Handle check boxes
//
public boolean action(Event evt, Object arg){
	boolean state;
	
	// Turn on/off lines (vs. points), tick marks, quality graph
	// (vs. quality numbers), and each of the four traces based 
	// on a change in a check box.  
	if (evt.target == LineCheck){
		theTraceCanvas.setLine(((Boolean)arg).booleanValue());
		theTraceCanvas.repaint();
	}
			
	if (evt.target == TickCheck){
		theTraceCanvas.setTick(((Boolean)arg).booleanValue());
		theTraceCanvas.repaint();
	}
	
	if (evt.target == GraphCheck){
		theQualityCanvas.setGraph(((Boolean)arg).booleanValue());
		theQualityCanvas.repaint();
	}
	
	if (evt.target == TempCheck){
		theQualityCanvas.setTemp(((Boolean)arg).booleanValue());
		theQualityCanvas.repaint();
	}
			
	if (evt.target == ACheck){
		theTraceCanvas.setA(((Boolean)arg).booleanValue());
		theTraceCanvas.repaint();
	}
	if (evt.target == CCheck){
		theTraceCanvas.setC(((Boolean)arg).booleanValue());
		theTraceCanvas.repaint();
	}
	if (evt.target == GCheck){
		theTraceCanvas.setG(((Boolean)arg).booleanValue());
		theTraceCanvas.repaint();
	
	}
	if (evt.target == TCheck){
		theTraceCanvas.setT(((Boolean)arg).booleanValue());
		theTraceCanvas.repaint();
	}
	
	return(true);
}


} //------| End class TracePanel |------------------------------------------------

