//================================================================================
// TraceStream
//--------------------------------------------------------------------------------
//
// Encapsulates data and methods involved in opening a file (actually a URL), 
// creating a Trace object (SCFTrace or ABITrace as appropriate), and reading 
// trace data from the file.  
//
//================================================================================
//
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//       1         2         3         4         5         6         7         8

import java.io.*;
import java.net.*;

import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.zip.*;

import Trace;
import SCFTrace;
import PhredQuality;

//import ABITrace;

//------------------------------------------------------------------------------
// * TraceStream
//------------------------------------------------------------------------------
//
public class TraceStream {

Trace theTrace;

//------------------------------------------------------
// * GetMagicNum
//------------------------------------------------------
//
public int GetMagicNum(BufferedInputStream fileIn)
throws IOException
{
	// remember where we are
	fileIn.mark(8);
	
	// pass stream through a filter for basic data types (int, float,etc.)
	DataInputStream DataFileIn = new DataInputStream(fileIn);
	int magicNum = DataFileIn.readInt();
	
	// Restore file pointer to original state.
	fileIn.reset();   
	
	return(magicNum);
}


//------------------------------------------------------
// * TraceStream
//------------------------------------------------------
// Constructor used by applications passing in directory and file names. 
//
public TraceStream(String tracePath,String qualPath)
throws IOException
{	
	File traceFile = new File(tracePath);
	FileInputStream traceFileIn = new FileInputStream(traceFile);

	// If it appears to be a .gz file... unzip it. 
	if (tracePath.endsWith(".gz")){
		 GZIPInputStream gzTraceIn = new GZIPInputStream(traceFileIn);	
		 BufferedInputStream traceIn = new BufferedInputStream(gzTraceIn);
		 theTrace = new SCFTrace(traceIn);
	}else{
		 BufferedInputStream traceIn = new BufferedInputStream(traceFileIn);
		 theTrace = new SCFTrace(traceIn);
	}

	if (qualPath != ""){
		 File qualFile = new File(qualPath);	
		 FileInputStream qualFileIn = new FileInputStream(qualFile);

		 if (qualPath.endsWith(".gz")){
			  GZIPInputStream gzQualFileIn = new GZIPInputStream(qualFileIn);
			  BufferedInputStream qualIn = new BufferedInputStream(gzQualFileIn);	
			  PhredQuality theQuality = new PhredQuality(qualIn);		
			  theTrace.CopyQuality(theQuality.Quality,theQuality.NumBases);
		 }else{
			  GZIPInputStream gzQualFileIn = new GZIPInputStream(qualFileIn);
			  BufferedInputStream qualIn = new BufferedInputStream(gzQualFileIn);	
			  PhredQuality theQuality = new PhredQuality(qualIn);		
			  theTrace.CopyQuality(theQuality.Quality,theQuality.NumBases);
		 }
	}else{
		 // Just to keep the data structure from being empty. 
	  if (theTrace.quality.length == 0){
		 theTrace.CopyBogusQuality();
	  }
	}		 
} 

//------------------------------------------------------
// * TraceStream
//------------------------------------------------------
// Constructor used by applications passing in directory and file names. 
//
public TraceStream(String traceDir,String traceName,String qualDir,String qualName)
throws IOException
{
	String tracePath = traceDir+traceName;
	String qualPath = qualDir+qualName;
	
	File traceFile = new File(tracePath);
	FileInputStream traceFileIn = new FileInputStream(traceFile);
	if (tracePath.endsWith(".gz")){
		 GZIPInputStream gzTraceIn = new GZIPInputStream(traceFileIn);	
		 BufferedInputStream traceIn = new BufferedInputStream(gzTraceIn);
		 theTrace = new SCFTrace(traceIn);
	}else{
		 BufferedInputStream traceIn = new BufferedInputStream(traceFileIn);
		 theTrace = new SCFTrace(traceIn);
	}

	if (qualName != ""){
		 File qualFile = new File(qualPath);	
		 FileInputStream qualFileIn = new FileInputStream(qualFile);

		 if (qualName.endsWith(".gz")){
			  GZIPInputStream gzQualFileIn = new GZIPInputStream(qualFileIn);
			  BufferedInputStream qualIn = new BufferedInputStream(gzQualFileIn);	
			  PhredQuality theQuality = new PhredQuality(qualIn);		
			  theTrace.CopyQuality(theQuality.Quality,theQuality.NumBases);
		 }else{
			  BufferedInputStream qualIn = new BufferedInputStream(qualFileIn);	
			  PhredQuality theQuality = new PhredQuality(qualIn);		
			  theTrace.CopyQuality(theQuality.Quality,theQuality.NumBases);
		 }
	}else{
	  if (theTrace.quality.length == 0){
		 theTrace.CopyBogusQuality();
	  }
	}
} 

//------------------------------------------------------
// * TraceStream
//------------------------------------------------------
// Constructor used by applets passing in document base URL's and file names. 
//
public TraceStream(URL DocumentBase,String traceName,String qualName)
throws IOException
{
	// Must create a URL to open the file since Netscape and other browsers do 
	// not allow reading and writing files on local file system.  I think this
	// implies that subsequent stream I/O goes through the web server where the
	// URL resides (taking a performance hit as it does).  I believe it also means
	// that everything accessed must be in a subdirectory (references work) of 
	// codeBase
	URL traceURL = new URL(DocumentBase, traceName);	

	if (traceName.endsWith(".gz")){
		 GZIPInputStream gzTraceIn = new GZIPInputStream(traceURL.openStream());
		 BufferedInputStream traceIn = new BufferedInputStream(gzTraceIn);
		 theTrace = new SCFTrace(gzTraceIn);
	}else{
		 BufferedInputStream traceIn = new BufferedInputStream(traceURL.openStream());
		 theTrace = new SCFTrace(traceIn);
	}

	if ((qualName != null) && (qualName != "")){
		 URL qualURL = new URL(DocumentBase, qualName);

		 if (qualName.endsWith(".gz")){
			  GZIPInputStream gzQualFileIn = new GZIPInputStream(qualURL.openStream());
			  BufferedInputStream qualIn = new BufferedInputStream(gzQualFileIn);
			  PhredQuality theQuality = new PhredQuality(qualIn);
			  theTrace.CopyQuality(theQuality.Quality,theQuality.NumBases);
		 }else{
			  BufferedInputStream qualIn = new BufferedInputStream(qualURL.openStream());
			  PhredQuality theQuality = new PhredQuality(qualIn);
			  theTrace.CopyQuality(theQuality.Quality,theQuality.NumBases);
		 }
	}else{
	  if (theTrace.quality.length == 0){
		 theTrace.CopyBogusQuality();
	  }
	}
} 




}//-------| End TraceStream class |-----------------------------------------------


