//==============================================================================
//  TraceViewerApp.java
//------------------------------------------------------------------------------
//
// TraceViewerApp provides a home for the TracePanel when run as an 
// application.  The frame provides a place for menus and the ability to 
// resize the frame/panel.  This class is not used when run as an applet 
// since the applet itself provides the home for the TracePanel.   
//
//
//==============================================================================

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;

import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import Trace;
import TraceStream;
import TracePanel;


// Since it's an application, may as well use a Swing frame..
public class TraceViewerApp extends Frame{
	 
	 TracePanel mTracePanel;	
	 AboutDialog mAboutDialog;

	 MenuItem miOpenFile = new MenuItem("Open SCF File...");
	 MenuItem miQuit = new MenuItem("Quit");
	 MenuItem miAbout = new MenuItem("About BCM Traceviewer..."); 
	 
	 //------------------------------------------------------
	 // * InstallMenu
	 //------------------------------------------------------
	 //
	 public void InstallMenu(){
		  MenuBar theBar = new MenuBar();
		  Menu  FileMenu = new Menu("File");
		  
		  this.setMenuBar(theBar);
		  theBar.add(FileMenu);
		  FileMenu.add(miAbout);
		  FileMenu.addSeparator();
		  FileMenu.add(miQuit);
	 }

	 //--------------------------------
	 // * TraceViewerApp
	 //--------------------------------
	 //	 
	 public TraceViewerApp(String SCFName,String QualName){

		  super("bcm-trace-view: "+SCFName);

		  addWindowListener(new WindowAdapter() {
					 public void windowClosing(WindowEvent e) {System.exit(0);}
				});
		
		  try {
				// TraceStream should throw an exception if fileName not valid or 
				// other problems occur.  If no problems occur, theStream should
				// contain a valid trace. 
				TraceStream theTraceStream=new TraceStream(SCFName,QualName);
     
				InstallTracePanel(theTraceStream.theTrace);	
		  }
		  catch (MalformedURLException ex) {
				System.err.println("Malformed URL");
		  } // end catch
		  
		  catch (IOException ex) {
				String exMessage = ex.getMessage();
				String exName = ex.getClass().getName();
				System.out.println("IO ERROR: " + exName + " : " + exMessage);
		  }

		  setVisible(true);
		  InstallMenu();
		  mAboutDialog = new AboutDialog(this,"BCM Trace Viewer");
		  pack();
	 }

	 //------------------------------------------------------
	 // * InstallTracePanel
	 //------------------------------------------------------
	 // Create the TracePanel and add it to the applet.  
	 //
	 public void InstallTracePanel(Trace theTrace)
	 {
		  int i=0;
		  mTracePanel = new TracePanel(theTrace);		  

		  // Make panel a little smaller than the 
		  //mTracePanel.resize(this.size().width-8,this.size().height-8);
		  mTracePanel.theTraceCanvas.resize(600,100);
		  mTracePanel.theQualityCanvas.resize(600,27);
		  //mTracePanel.theTraceCanvas.resize(this.size().width-100,this.size().height-100);
		  //mTracePanel.theQualityCanvas.resize(this.size().width-100,27);
		  
		  setLayout(new GridLayout());
		  add(mTracePanel);		  
		  
		  mTracePanel.FinishInit();
		  mTracePanel.validate();
		  
		  this.validate();
		  // System.out.println("InstallTracePane Applet width="+size().width+
		  // "height="+size().height);
		  // System.out.println("mTracePanel width="+mTracePanel.size().width+
		  // "height="+mTracePanel.size().height);
		  mTracePanel.resize(700,150);
		  mTracePanel.theTraceCanvas.resize(600,100);
		  mTracePanel.theQualityCanvas.resize(600,27);
		  mTracePanel.repaint();

		  setSize(700,200);
		  resize(700,200);
		  validate();
		  repaint();
	 }

	 //------------------------------------------------------
	 // * action
	 //------------------------------------------------------
	 // Handle menu actions
	 //
	 public boolean action(Event event,Object arg){
		  if (event.target == miOpenFile){				
				// Get file names from file picker dialog. 
				//if (GetNewFileInfo()){
					 
					 // Read the trace and quality from the specified file(s)
				//		 GetTraceFromFile();
					 
				//		 mTracePanel.setNewTrace(mTrace);
				//		 mTracePanel.repaint();
				//	}
		  }				  
		  if (event.target == miQuit){
				//dispose();
				System.exit(0);
		  }
		  
		  if (event.target == miAbout){
				mAboutDialog.show();
		  }		  
		  return(true);
	 }
	 
	 //--------------------------------
	 // * main
	 //--------------------------------
	 //
	 public static void main(String argv[]) 
	 {		  
		  // argv[0] is scf name
		  // argv[1] is quality name (optional)
		  if (argv.length < 1){
				System.err.println("Must supply an scf file name.");
		  }else{				
				if (argv.length == 1){
					 TraceViewerApp tv = new TraceViewerApp(argv[0],"");											   
				}else{
					 TraceViewerApp tv = new TraceViewerApp(argv[0],argv[1]);			
				}
		  }
	 }


}
